package org.dbis.xmlpraktikum;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.PrintWriter;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.log4j.Logger;
import org.jdom.Document;
import org.jdom.Element;
import org.jdom.JDOMException;
import org.jdom.input.SAXBuilder;

public class FormatServlet extends HttpServlet {

	private static final long serialVersionUID = -8644396054056558046L;

	private static final Logger logger = Logger.getLogger(FormatServlet.class);

	/**
	 * The collection "results" stores all generated HTML result tables. To
	 * avoid ConcurrentModificationExceptions, the collection is synchronized
	 * upon creation.
	 */
	private static Collection<String> results = Collections
			.synchronizedCollection(new ArrayList<String>());

	@Override
	protected void doPost(HttpServletRequest req, HttpServletResponse resp)
			throws ServletException, IOException {

		BufferedReader in = new BufferedReader(new InputStreamReader(req
				.getInputStream()));

		SAXBuilder builder = new SAXBuilder();
		Document doc;

		// build the document
		try {
			doc = builder.build(in);
		} catch (JDOMException e) {
			logger.error("could not build xml request", e);
			resp.sendError(HttpServletResponse.SC_BAD_REQUEST, e.getMessage());
			return;
		}

		Element root = doc.getRootElement();

		// get the parameters
		String a = root.getChildTextTrim("a");
		String b = root.getChildTextTrim("b");
		String result = root.getChildTextTrim("result");

		if (a == null || b == null || result == null) {
			logger.error("xml not valid");
			resp.sendError(HttpServletResponse.SC_BAD_REQUEST, "xml not valid");
			return;
		}

		// create the response
		String resultTable = createResultTable(a, b, result);

		PrintWriter out = resp.getWriter();

		out.print(resultTable);
		out.flush();
		out.close();

		results.add(resultTable);
	}

	@Override
	protected void doGet(HttpServletRequest req, HttpServletResponse resp)
			throws ServletException, IOException {

		PrintWriter out = resp.getWriter();

		out.print(createDocumentHeader("Generated Result Tables"));
		out.print("<h1>Generated Result Tables:</h1>\n");

		for (String resultTable : results) {
			out.print(resultTable);
			out.print("\n<br/>\n");
		}

		out.print(createDocumentFooter());

		out.flush();
		out.close();
	}

	private static final String createResultTable(String a, String b,
			String result) {
		StringBuilder sb = new StringBuilder();

		sb.append("<table border=\"1\">\n");
		sb.append("  <tr><th>a</th><th>b</th><th>result (a + b)</th></tr>\n");
		sb.append("  <tr>\n");
		sb.append("    <td>").append(a).append("</td>\n");
		sb.append("    <td>").append(b).append("</td>\n");
		sb.append("    <td>").append(result).append("</td>\n");
		sb.append("  </tr>\n");
		sb.append("</table>");

		return sb.toString();
	}

	private String createDocumentHeader(String title) {
		StringBuilder sb = new StringBuilder();
		sb.append("<!DOCTYPE HTML PUBLIC ").append(
				"\"-//W3C//DTD HTML 4.0 Transitional//EN\">\n");
		sb.append("<html>\n<head><title>").append(title).append(
				"</title></head>\n").append("<body>\n");

		return sb.toString();
	}

	private String createDocumentFooter() {
		return "</body></html>\n";
	}
}
